#include <glm/gtc/type_ptr.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <imgui.h>
#include <imgui_internal.h>
#include <nfd.h>

#include "../../libs/guizmo/ImGuizmo.h"
#include "../../YoggieEngine/src/BarinkEngine.h"
#include "../../YoggieEngine/src/AssetManager/ModelImporter.h"
#include "../../YoggieEngine/src/Graphics/Memory/Framebuffer.h"
#include "../../YoggieEngine/src/PerfCounter.cpp"
#include "../../YoggieEngine/src/Scene/Entity.h"
#include "UI/Widgets.h"
#include "Project.h"
#include "SceneSerializer.h"



using namespace YoggieEngine;
/*
* Define globals
*/

struct EditorContext  {
    Project CurrentProject;
    Scene MainScene;

    EditorContext() = default;
    EditorContext(EditorContext& other) = default;
    ~EditorContext() = default;
};

std::shared_ptr<Project> CurrentProject;
Scene MainScene;

Framebuffer* framebuffer;
SceneObject* Model;
Entity cube;


entt::entity  Selected;

/*
* Runs once at startup
* - USe to initialize the game/sandbox/demo
*/
void Start() {

    CurrentProject = std::make_shared<Project>("Random");
    auto io = ImGui::GetIO();
    io.Fonts->AddFontFromFileTTF("build/Debug/Fonts/Roboto-Regular.ttf", 18);
 
    framebuffer = new Framebuffer();


    // Create a level and load it as the current level
    auto importer = ModelImporter();
    
    // Create a cube 
    Model = importer.Import("build/Debug/Models/Cube.obj");
    cube = MainScene.AddEntity("cube");
    
    auto& render3DComponent = cube.AddComponent<Render3DComponent>();
    render3DComponent.mesh = *(Model->renderable->mesh);

    cube.GetComponent<TransformComponent>().Position = glm::vec3(1.0f, 0.0f, 5.0f);
    
    auto cube2 = MainScene.AddEntity("Cube1");
    auto& rendercube2 = cube2.AddComponent<Render3DComponent>();
    rendercube2.mesh = *(Model->renderable->mesh);


    // create an ambient light source
    auto AmbientLight = MainScene.AddEntity("AmbientLight");
    auto light = AmbientLight.AddComponent<LightComponent>();
    light.Color = glm::vec3(1.0f);
    light.Strength = 1.0f;

    Selected = (entt::entity) -1;

    renderer.Prepare(MainScene);
}


char* path = nullptr;
char* savePath = nullptr;
char* scenePath = nullptr;
char* openScenePath = nullptr;
/*
* Runs every frame
* - Use to draw Immediate mode graphics (Not meant for HUD's )
*/
void ImmediateGraphicsDraw()
{   ImGui::DockSpaceOverViewport(ImGui::GetMainViewport());
     
    // Show a menu bar

    ImGui::BeginMainMenuBar();

    if (ImGui::BeginMenu("Application")) {

        if (ImGui::MenuItem("Load Project")) 
        {
            nfdresult_t result = NFD_OpenDialog({ "yproj" }, NULL, &path);
                switch (result) {
                case(NFD_OKAY):
                    Project::LoadProject( path, CurrentProject);
                    break;
                case(NFD_CANCEL):
                    break;
                case(NFD_ERROR):
                    std::cout << "NFD_Error: " << NFD_GetError() << std::endl;
                    break;
            }

        }

        if (ImGui::MenuItem("Save project as...")) {
            nfdresult_t result = NFD_SaveDialog({ "yproj" }, NULL, &savePath);
            switch (result) {
            case(NFD_OKAY):
                std::cout << "Save as: " << savePath << std::endl;
                Project::SaveProject(savePath, *CurrentProject.get());
                break;
            case(NFD_CANCEL):
                break;
            case(NFD_ERROR):
                std::cout << "NFD_Error: " << NFD_GetError() << std::endl;
                break;
            }
        }



        if (ImGui::MenuItem("Preferences"))
        {

        }

        if (ImGui::MenuItem("Exit"))
        {
            // TODO: Exit application
        }

        ImGui::EndMenu();
    }


    if (ImGui::BeginMenu("Scene")) {

        if (ImGui::MenuItem("Save scene"))
        {
            nfdresult_t result= NFD_SaveDialog({"yscene"},NULL, &scenePath);
            switch (result) {
            case(NFD_OKAY):
                SaveScene(scenePath, MainScene);
                break;
            case(NFD_CANCEL):
                break;
            case(NFD_ERROR):
                std::cout << "NFD_Error: " << NFD_GetError() << std::endl;
                break;
            }

        }

        if (ImGui::MenuItem("Load scene"))
        {
            auto result = NFD_OpenDialog({ "yscene" }, NULL, &openScenePath);
            switch (result) {
            case (NFD_OKAY):
                LoadScene(openScenePath, MainScene);
                break;
            case(NFD_CANCEL):
                break;
            case(NFD_ERROR):
                std::cout << "NFD_Error: " << NFD_GetError() << std::endl;
                break;
            }

        }

        if (ImGui::MenuItem("Add Entity")) {
            MainScene.AddEntity("New entity");
        }

        ImGui::EndMenu();
    }


    ImGui::EndMainMenuBar();

    ImGui::Begin("ProjectInfo");
    ImGui::Text("Project: %s", CurrentProject.get()->GetName().c_str());
    ImGui::Text("Directory: %s", CurrentProject.get()->GetProjectDirectory().u8string().c_str());
    ImGui::End();


    //ShowStats();
    Viewport(*framebuffer, MainScene);
    SceneExplorer(Selected, MainScene);
    Inspector(Selected, MainScene );

    Settings();
    AssetsFinder();
    Console();

    ImGui::ShowDemoWindow();
    ImGui::ShowMetricsWindow();
   

}

void Render() 
{
    renderer.Render( *framebuffer, MainScene);
}

/*
* Runs every frame
* - Meant for game logic ( non-physics related)
*/
void Update()
{
}

/*
* Runs every physics update
*/
void fixed_update() 
{
    
}

/*
* Runs at the end of the program
* - Meant for cleanup
*/
void Stop()
{
    delete framebuffer;
}