#include "BarinkEngine.h"
using namespace YoggieEngine;

Renderer renderer;

const unsigned int MS_PER_UPDATE = 2;

int main(int argc, char* argv[]) {
	// Setup performance sampler 
	EngineInstrumentation::PerfomanceSamplerInit();

	// Startup services 
	BarinkWindow MainWindow = BarinkWindow(1200, 700);

	renderer = Renderer();
	InputSystem = InputManager();

	InputSystem.attach(&MainWindow);

	GUIManager GUISystem = GUIManager(&MainWindow);

	glEnable(GL_DEPTH_TEST);

	// First call to setup game
	{
		Start();
	}


	double previous = glfwGetTime();
	double lag = 0.0;

	// Runtime loop
	while (!MainWindow.WindowShouldClose())
	{

		double current = glfwGetTime();
		double elapsed = current - previous;
		previous = current;
		lag += elapsed;

		//EngineInstrumentation::Update(); // Todo this does nothing right now and is therefor disabled

		// Execute main logic 
		{
			InputSystem.PollEvents();

		}

		{
			while (lag >= MS_PER_UPDATE) {
				Update();
				lag -= MS_PER_UPDATE;
			}
		}

		{
			Render();
		}

		{
			GUISystem.Render();
		}


		{
			MainWindow.SwapBuffers();
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		}

	}

	// Shutdown game
	{
		Stop();
	}

	// Shutdown Services

	return 0;
}



